<?php

/**
 * @file
 * Contains \Drupal\editablevar\EditablevarVarUpdateForm
 */

namespace Drupal\editablevar;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Utility\Xss; 
use Drupal\Core\StringTranslation\TranslatableMarkup;  


/**
 * Simple form to update an entry, with all the interesting fields.
 */
class EditablevarVarUpdateForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormID() {
    return 'editablevar_var_update_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $editablevar_var_id = 0) {
    $var = EditablevarVarStorage::loadById($editablevar_var_id);
    if (!$var) {
      return array('#markup' => $this->t('Failed to load this variable'));
    }
    $group = NULL;
    // load all groups to build the options for the "move to another group" functionality
    $move_group_options=array();
    foreach ($groups = EditablevarGroupStorage::load() as $param_group) {
      // find the group of the given variable
      if ($param_group->record_id==$var->group_record_id) {
        $group=$param_group;
      }
      else {
        // collect the other groups for the "move" select element
        // Use "Groupname: prefix_varname" format as option text,
        // add "group name" and "prefix" here, and "varname" later
        $move_group_options[$param_group->record_id]=$this->t('Move to @group: @var', ['@group' => $param_group->name, '@var' => $param_group->var_id_prefix]);
      }
    }
    if (!$group) {
      return array('#markup' => $this->t('Failed to load this group'));
    }
    $form = array();
    $form['var_record_id'] = array(
      '#type' => 'value',
      '#value' => $var->record_id,
    );
    $form['var_id_for_delete'] = array(
      '#type' => 'value',
      '#value' => $var->id,
    );
    $form['var_id_prefix'] = array(
      '#type' => 'value',
      '#value' => $group->var_id_prefix,
    );

    $form['group_record_id'] = array(
      '#type' => 'value',
      '#value' => $var->group_record_id,
    );

    $form['update'] = array(
      '#type' => 'fieldset',
      '#title' => t('Update variable'),
    );
    $id_without_prefix = $var->id;
    if ($group->var_id_prefix!='') {
      if (substr($var->id, 0, strlen($group->var_id_prefix))==$group->var_id_prefix) {
        $id_without_prefix = substr($var->id, strlen($group->var_id_prefix));
      }
      else {
        $form['update']['id_prefix_message'] = array(
          '#markup' => '<div class="messages">' . $this->t('The group prefix "@prefix" will be added to the name of this variable when the variable is saved.', array('@prefix' => $group->var_id_prefix)) . '</div>',
        );
      }
    }
    if ($move_group_options) {
      // Use "Groupname: prefix_varname" format as option text,
      // add "group name" and "prefix" earlier, and "varname" here
      foreach ($move_group_options as $key => $value) {
        $move_group_options[$key]=$value . $id_without_prefix;
      }
      $form['update']['move_to_group_record_id'] = array(
        '#type' => 'select',
        '#title' => $this->t('Group'),
        '#options' => array($group->record_id => $group->name) + $move_group_options,
        '#default_value' => $group->record_id,
        '#required' => TRUE,
      );
    }
    else {
      $form['move_to_group_record_id'] = array(
        '#type' => 'value',
        '#value' => $group->record_id,
      );
    }
    $form['update']['id'] = array(
      '#type' => 'machine_name',
      '#title' => $this->t('Name'),
      '#maxlength' => 64,
      '#description' => $this->t('A unique name for this variable. Must be alpha-numeric and underscore separated.'),
      '#default_value' => $id_without_prefix,
      '#field_prefix' => $group->var_id_prefix,
      '#machine_name' => array(
        'exists' => array($this, 'exists'),
      ),
      '#required' => TRUE,
      '#disabled' => FALSE,
    );
    $form['update']['description'] = array(
      '#type' => 'textarea',
      '#title' => t('Description'),
      '#default_value' => $var->description,
    );
    $form['update']['value'] = array(
      '#type' => 'textarea',
      '#title' => t('Value'),
      '#default_value' => $var->value,
    );
    $form['actions'] = array(
      '#type' => 'actions',
      '#attributes' => array('class' => array('container-inline')),
    );
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    );
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Delete'),
      '#prefix' => '&nbsp;&nbsp;&nbsp;',
      '#submit' => array('::deleteForm'),
    );
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $id = $form_state->getValue('var_id_prefix') . $form_state->getValue('id');
    if (strlen($id)>=100) {
      $form_state->setErrorByName('id', $this->t('Variable name must be less than 100 characters long.'));
    }
    // Check whether variable should be moved
    if ($form_state->getValue('move_to_group_record_id') != $form_state->getValue('group_record_id')) {
      $found=FALSE;
      foreach ($groups = EditablevarGroupStorage::load(array('record_id' => $form_state->getValue('move_to_group_record_id'))) as $new_group) {
        $found=TRUE;
        // use the prefix of the new group
        $moved_id=$new_group->var_id_prefix . $form_state->getValue('id');
        // check whether a variable exists with the same in the destination group
        // @see $this->exists()
        $database = \Drupal::database();
        // Read entry with the given ID
        $select = $database->select('editablevar_vars', 'vars');
        $select->fields('vars');
        $select->condition('id', $moved_id);
        // Check the result as boolean
        if ((bool) $select->execute()->fetchAll()) {
          $form_state->setErrorByName('move_to_group_record_id', $this->t('The variable cannot be moved to the @group group. The machine-readable name @moved_id is already in use. It must be unique.', ['@group' => $new_group->name, '@moved_id' => $moved_id]));
        }
      }
      if (!$found) {
        $form_state->setErrorByName('move_to_group_record_id', $this->t('The destination group does not exist. Reload the page and try again.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save the submitted entry.
    $id = $form_state->getValue('var_id_prefix') . $form_state->getValue('id');
    $entry = array(
      'record_id' => $form_state->getValue('var_record_id'),
      'id' => $id,
      'description' => trim($form_state->getValue('description')),
      'value' => trim($form_state->getValue('value')),
    );
    // Check whether variable should be moved
    if ($form_state->getValue('move_to_group_record_id') != $form_state->getValue('group_record_id')) {
      foreach ($groups = EditablevarGroupStorage::load(array('record_id' => $form_state->getValue('move_to_group_record_id'))) as $new_group) {
        // use the prefix of the new group
        $moved_id=$new_group->var_id_prefix . $form_state->getValue('id');
        \Drupal::messenger()->addStatus(t('Moving variable @id to group @group: @moved_id', array('@id' => $id, '@moved_id' => $moved_id, '@group' => $new_group->name)));
        $entry['group_record_id']=$new_group->record_id;
        $entry['id']=$moved_id;
        break;
      }
    }
  
    
    $return = EditablevarVarStorage::update($entry);
    if ($return) {
      \Drupal::messenger()->addStatus(t('Updated variable @variable', array('@variable' => $id)));
      foreach ($groups = EditablevarGroupStorage::load(array('record_id' => $form_state->getValue('group_record_id'))) as $group) {
        $form_state->setRedirectUrl(new Url('editablevar.var_list', array('editablevar_group_id' => $group->id)));
      }
    }
  }
  /**
   * {@inheritdoc}
   */
  public function deleteForm(array &$form, FormStateInterface $form_state) {
    $form_state->setRedirectUrl(new Url('editablevar.var_delete', array('editablevar_var_id' => $form_state->getValue('var_id_for_delete'))));
  }
  public function exists($id, array $element, FormStateInterface $form_state) {
    // Check whether variable should be moved to another group
    if ($form_state->getValue('move_to_group_record_id') != $form_state->getValue('group_record_id')) {
      // When a variable is moved to another group, the validateForm()
      // checks the machine name, taking into account the prefix of the
      // destination group. Therefore the machine name checking
      // of the source group is skipped.
      return FALSE;
    }
    $database = \Drupal::database();
    // Read entry with the given ID
    $select = $database->select('editablevar_vars', 'vars');
    $select->fields('vars');
    $select->condition('id', $element['#field_prefix'] . $id);
    // Return the result as boolean
    return (bool) $select->execute()->fetchAll();
  }
}
